"""ActionBatchOrganizations API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

import urllib.parse
from typing import Any

from meraki_client.schemas import (
    BulkUpdateOrganizationDevicesDetailsDetailsItem,
    ClaimOrganizationInventoryOrdersSubscriptionsItem,
    CreateOrganizationActionBatchActionsItem,
    CreateOrganizationAdaptivePolicyAclRulesItem,
    CreateOrganizationAdaptivePolicyGroupPolicyObjectsItem,
    CreateOrganizationAdaptivePolicyPolicyAclsItem,
    CreateOrganizationAdaptivePolicyPolicyDestinationGroup,
    CreateOrganizationAdaptivePolicyPolicySourceGroup,
    CreateOrganizationAlertsProfileAlertCondition,
    CreateOrganizationAlertsProfileRecipients,
    CreateOrganizationBrandingPolicyAdminSettings,
    CreateOrganizationBrandingPolicyCustomLogo,
    CreateOrganizationBrandingPolicyHelpSettings,
    CreateOrganizationDevicesPacketCaptureScheduleDevicesItem,
    CreateOrganizationDevicesPacketCaptureScheduleSchedule,
    DisableOrganizationIntegrationsXdrNetworksNetworksItem,
    EnableOrganizationIntegrationsXdrNetworksNetworksItem,
    ReorderOrganizationDevicesPacketCaptureSchedulesOrderItem,
    UpdateOrganizationAdaptivePolicyAclRulesItem,
    UpdateOrganizationAdaptivePolicyGroupPolicyObjectsItem,
    UpdateOrganizationAdaptivePolicyPolicyAclsItem,
    UpdateOrganizationAdaptivePolicyPolicyDestinationGroup,
    UpdateOrganizationAdaptivePolicyPolicySourceGroup,
    UpdateOrganizationAlertsProfileAlertCondition,
    UpdateOrganizationAlertsProfileRecipients,
    UpdateOrganizationBrandingPolicyAdminSettings,
    UpdateOrganizationBrandingPolicyCustomLogo,
    UpdateOrganizationBrandingPolicyHelpSettings,
    UpdateOrganizationDevicesPacketCaptureScheduleDevicesItem,
    UpdateOrganizationDevicesPacketCaptureScheduleSchedule,
    UpdateOrganizationLoginSecurityApiAuthentication,
)
from meraki_client.types import (
    CreateOrganizationAdaptivePolicyAclIpVersion,
    CreateOrganizationAdaptivePolicyPolicyLastEntryRule,
    CreateOrganizationAlertsProfileType,
    CreateOrganizationDevicesControllerMigrationTarget,
    CreateOrganizationNetworkProductTypes,
    UpdateOrganizationAdaptivePolicyAclIpVersion,
    UpdateOrganizationAdaptivePolicyPolicyLastEntryRule,
    UpdateOrganizationAlertsProfileType,
)


class ActionBatchOrganizations:
    """ActionBatchOrganizations class."""

    def __init__(self) -> None:
        pass

    def create_organization_adaptive_policy_acl(
        self,
        *,
        organization_id: str,
        name: str,
        rules: list[CreateOrganizationAdaptivePolicyAclRulesItem],
        ip_version: CreateOrganizationAdaptivePolicyAclIpVersion,
        description: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Creates new adaptive policy ACL.

        [API documentation: createOrganizationAdaptivePolicyAcl](https://developer.cisco.com/meraki/api-v1/#!create-organization-adaptive-policy-acl)

        Args:
            organization_id: Organization ID.
            name: Name of the adaptive policy ACL.
            description: Description of the adaptive policy ACL.
            rules: An ordered array of the adaptive policy ACL rules.
            ip_version: IP version of adpative policy ACL. One of: 'any', 'ipv4' or 'ipv6'.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/acls"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if description is not None:
            payload["description"] = description
        if rules is not None:
            payload["rules"] = [item.model_dump(by_alias=True, exclude_none=True) for item in rules]
        if ip_version is not None:
            payload["ipVersion"] = ip_version

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_adaptive_policy_acl(
        self,
        *,
        organization_id: str,
        acl_id: str,
        name: str | None = None,
        description: str | None = None,
        rules: list[UpdateOrganizationAdaptivePolicyAclRulesItem] | None = None,
        ip_version: UpdateOrganizationAdaptivePolicyAclIpVersion | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates an adaptive policy ACL.

        [API documentation: updateOrganizationAdaptivePolicyAcl](https://developer.cisco.com/meraki/api-v1/#!update-organization-adaptive-policy-acl)

        Args:
            organization_id: Organization ID.
            acl_id: Acl ID.
            name: Name of the adaptive policy ACL.
            description: Description of the adaptive policy ACL.
            rules: An ordered array of the adaptive policy ACL rules. An empty array will clear the
                rules.
            ip_version: IP version of adpative policy ACL. One of: 'any', 'ipv4' or 'ipv6'.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        acl_id = urllib.parse.quote(str(acl_id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/acls/{acl_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if description is not None:
            payload["description"] = description
        if rules is not None:
            payload["rules"] = [item.model_dump(by_alias=True, exclude_none=True) for item in rules]
        if ip_version is not None:
            payload["ipVersion"] = ip_version

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_adaptive_policy_acl(
        self, *, organization_id: str, acl_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes the specified adaptive policy ACL.

        [API documentation: deleteOrganizationAdaptivePolicyAcl](https://developer.cisco.com/meraki/api-v1/#!delete-organization-adaptive-policy-acl)

        Args:
            organization_id: Organization ID.
            acl_id: Acl ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        acl_id = urllib.parse.quote(str(acl_id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/acls/{acl_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_adaptive_policy_group(
        self,
        *,
        organization_id: str,
        name: str,
        sgt: int,
        description: str | None = None,
        policy_objects: list[CreateOrganizationAdaptivePolicyGroupPolicyObjectsItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Creates a new adaptive policy group.

        [API documentation: createOrganizationAdaptivePolicyGroup](https://developer.cisco.com/meraki/api-v1/#!create-organization-adaptive-policy-group)

        Args:
            organization_id: Organization ID.
            name: Name of the group.
            sgt: SGT value of the group.
            description: Description of the group (default: "").
            policy_objects: The policy objects that belong to this group; traffic from addresses
                specified by these policy objects will be tagged with this group's SGT
                value if no other tagging scheme is being used (each requires one unique
                attribute) (default: []).

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/groups"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if sgt is not None:
            payload["sgt"] = sgt
        if description is not None:
            payload["description"] = description
        if policy_objects is not None:
            payload["policyObjects"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in policy_objects
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_adaptive_policy_group(
        self,
        *,
        organization_id: str,
        id: str,
        name: str | None = None,
        sgt: int | None = None,
        description: str | None = None,
        policy_objects: list[UpdateOrganizationAdaptivePolicyGroupPolicyObjectsItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates an adaptive policy group.

        [API documentation: updateOrganizationAdaptivePolicyGroup](https://developer.cisco.com/meraki/api-v1/#!update-organization-adaptive-policy-group)

        Args:
            organization_id: Organization ID.
            id: ID.
            name: Name of the group.
            sgt: SGT value of the group.
            description: Description of the group.
            policy_objects: The policy objects that belong to this group; traffic from addresses
                specified by these policy objects will be tagged with this group's SGT
                value if no other tagging scheme is being used (each requires one unique
                attribute).

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/groups/{id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if sgt is not None:
            payload["sgt"] = sgt
        if description is not None:
            payload["description"] = description
        if policy_objects is not None:
            payload["policyObjects"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in policy_objects
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_adaptive_policy_group(
        self, *, organization_id: str, id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes the specified adaptive policy group and any associated policies and references.

        [API documentation: deleteOrganizationAdaptivePolicyGroup](https://developer.cisco.com/meraki/api-v1/#!delete-organization-adaptive-policy-group)

        Args:
            organization_id: Organization ID.
            id: ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/groups/{id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_adaptive_policy_policy(
        self,
        *,
        organization_id: str,
        source_group: CreateOrganizationAdaptivePolicyPolicySourceGroup,
        destination_group: CreateOrganizationAdaptivePolicyPolicyDestinationGroup,
        acls: list[CreateOrganizationAdaptivePolicyPolicyAclsItem] | None = None,
        last_entry_rule: CreateOrganizationAdaptivePolicyPolicyLastEntryRule | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Add an Adaptive Policy.

        [API documentation: createOrganizationAdaptivePolicyPolicy](https://developer.cisco.com/meraki/api-v1/#!create-organization-adaptive-policy-policy)

        Args:
            organization_id: Organization ID.
            source_group: The source adaptive policy group (requires one unique attribute).
            destination_group: The destination adaptive policy group (requires one unique
                attribute).
            acls: An ordered array of adaptive policy ACLs (each requires one unique attribute) that
                apply to this policy (default: []).
            last_entry_rule: The rule to apply if there is no matching ACL (default: "default").

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/policies"

        payload: dict[str, Any] = {}
        if source_group is not None:
            payload["sourceGroup"] = source_group.model_dump(by_alias=True, exclude_none=True)
        if destination_group is not None:
            payload["destinationGroup"] = destination_group.model_dump(
                by_alias=True, exclude_none=True
            )
        if acls is not None:
            payload["acls"] = [item.model_dump(by_alias=True, exclude_none=True) for item in acls]
        if last_entry_rule is not None:
            payload["lastEntryRule"] = last_entry_rule

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_adaptive_policy_policy(
        self,
        *,
        organization_id: str,
        id: str,
        source_group: UpdateOrganizationAdaptivePolicyPolicySourceGroup | None = None,
        destination_group: UpdateOrganizationAdaptivePolicyPolicyDestinationGroup | None = None,
        acls: list[UpdateOrganizationAdaptivePolicyPolicyAclsItem] | None = None,
        last_entry_rule: UpdateOrganizationAdaptivePolicyPolicyLastEntryRule | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update an Adaptive Policy.

        [API documentation: updateOrganizationAdaptivePolicyPolicy](https://developer.cisco.com/meraki/api-v1/#!update-organization-adaptive-policy-policy)

        Args:
            organization_id: Organization ID.
            id: ID.
            source_group: The source adaptive policy group (requires one unique attribute).
            destination_group: The destination adaptive policy group (requires one unique
                attribute).
            acls: An ordered array of adaptive policy ACLs (each requires one unique attribute) that
                apply to this policy.
            last_entry_rule: The rule to apply if there is no matching ACL.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/policies/{id}"

        payload: dict[str, Any] = {}
        if source_group is not None:
            payload["sourceGroup"] = source_group.model_dump(by_alias=True, exclude_none=True)
        if destination_group is not None:
            payload["destinationGroup"] = destination_group.model_dump(
                by_alias=True, exclude_none=True
            )
        if acls is not None:
            payload["acls"] = [item.model_dump(by_alias=True, exclude_none=True) for item in acls]
        if last_entry_rule is not None:
            payload["lastEntryRule"] = last_entry_rule

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_adaptive_policy_policy(
        self, *, organization_id: str, id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete an Adaptive Policy.

        [API documentation: deleteOrganizationAdaptivePolicyPolicy](https://developer.cisco.com/meraki/api-v1/#!delete-organization-adaptive-policy-policy)

        Args:
            organization_id: Organization ID.
            id: ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/policies/{id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def update_organization_adaptive_policy_settings(
        self, organization_id: str, *, enabled_networks: list[str] | None = None
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update global adaptive policy settings.

        [API documentation: updateOrganizationAdaptivePolicySettings](https://developer.cisco.com/meraki/api-v1/#!update-organization-adaptive-policy-settings)

        Args:
            organization_id: Organization ID.
            enabled_networks: List of network IDs with adaptive policy enabled.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/adaptivePolicy/settings"

        payload: dict[str, Any] = {}
        if enabled_networks is not None:
            payload["enabledNetworks"] = enabled_networks

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_organization_alerts_profile(
        self,
        *,
        organization_id: str,
        type_: CreateOrganizationAlertsProfileType,
        alert_condition: CreateOrganizationAlertsProfileAlertCondition,
        recipients: CreateOrganizationAlertsProfileRecipients,
        network_tags: list[str],
        description: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create an organization-wide alert configuration.

        [API documentation: createOrganizationAlertsProfile](https://developer.cisco.com/meraki/api-v1/#!create-organization-alerts-profile)

        Args:
            organization_id: Organization ID.
            type_: The alert type.
            alert_condition: The conditions that determine if the alert triggers.
            recipients: List of recipients that will recieve the alert.
            network_tags: Networks with these tags will be monitored for the alert.
            description: User supplied description of the alert.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/alerts/profiles"

        payload: dict[str, Any] = {}
        if type_ is not None:
            payload["type"] = type_
        if alert_condition is not None:
            payload["alertCondition"] = alert_condition.model_dump(by_alias=True, exclude_none=True)
        if recipients is not None:
            payload["recipients"] = recipients.model_dump(by_alias=True, exclude_none=True)
        if network_tags is not None:
            payload["networkTags"] = network_tags
        if description is not None:
            payload["description"] = description

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_alerts_profile(
        self,
        *,
        organization_id: str,
        alert_config_id: str,
        enabled: bool | None = None,
        type_: UpdateOrganizationAlertsProfileType | None = None,
        alert_condition: UpdateOrganizationAlertsProfileAlertCondition | None = None,
        recipients: UpdateOrganizationAlertsProfileRecipients | None = None,
        network_tags: list[str] | None = None,
        description: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update an organization-wide alert config.

        [API documentation: updateOrganizationAlertsProfile](https://developer.cisco.com/meraki/api-v1/#!update-organization-alerts-profile)

        Args:
            organization_id: Organization ID.
            alert_config_id: Alert config ID.
            enabled: Is the alert config enabled.
            type_: The alert type.
            alert_condition: The conditions that determine if the alert triggers.
            recipients: List of recipients that will recieve the alert.
            network_tags: Networks with these tags will be monitored for the alert.
            description: User supplied description of the alert.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        alert_config_id = urllib.parse.quote(str(alert_config_id), safe="")
        path = f"/organizations/{organization_id}/alerts/profiles/{alert_config_id}"

        payload: dict[str, Any] = {}
        if enabled is not None:
            payload["enabled"] = enabled
        if type_ is not None:
            payload["type"] = type_
        if alert_condition is not None:
            payload["alertCondition"] = alert_condition.model_dump(by_alias=True, exclude_none=True)
        if recipients is not None:
            payload["recipients"] = recipients.model_dump(by_alias=True, exclude_none=True)
        if network_tags is not None:
            payload["networkTags"] = network_tags
        if description is not None:
            payload["description"] = description

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_alerts_profile(
        self, *, organization_id: str, alert_config_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Removes an organization-wide alert config.

        [API documentation: deleteOrganizationAlertsProfile](https://developer.cisco.com/meraki/api-v1/#!delete-organization-alerts-profile)

        Args:
            organization_id: Organization ID.
            alert_config_id: Alert config ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        alert_config_id = urllib.parse.quote(str(alert_config_id), safe="")
        path = f"/organizations/{organization_id}/alerts/profiles/{alert_config_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_branding_policy(
        self,
        *,
        organization_id: str,
        name: str,
        enabled: bool | None = None,
        admin_settings: CreateOrganizationBrandingPolicyAdminSettings | None = None,
        help_settings: CreateOrganizationBrandingPolicyHelpSettings | None = None,
        custom_logo: CreateOrganizationBrandingPolicyCustomLogo | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Add a new branding policy to an organization.

        [API documentation: createOrganizationBrandingPolicy](https://developer.cisco.com/meraki/api-v1/#!create-organization-branding-policy)

        Args:
            organization_id: Organization ID.
            name: Name of the Dashboard branding policy.
            enabled: Boolean indicating whether this policy is enabled.
            admin_settings: Settings for describing which kinds of admins this policy applies to.
            help_settings: Settings for describing the modifications to various Help page features.
                Each property in this object accepts one of 'default or inherit' (do not
                modify functionality), 'hide' (remove the section from Dashboard), or
                'show' (always show the section on Dashboard). Some properties in this
                object also accept custom HTML used to replace the section on Dashboard;
                see the documentation for each property to see the allowed values. Each
                property defaults to 'default or inherit' when not provided.
            custom_logo: Properties describing the custom logo attached to the branding policy.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/brandingPolicies"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if enabled is not None:
            payload["enabled"] = enabled
        if admin_settings is not None:
            payload["adminSettings"] = admin_settings.model_dump(by_alias=True, exclude_none=True)
        if help_settings is not None:
            payload["helpSettings"] = help_settings.model_dump(by_alias=True, exclude_none=True)
        if custom_logo is not None:
            payload["customLogo"] = custom_logo.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_branding_policies_priorities(
        self, organization_id: str, *, branding_policy_ids: list[str] | None = None
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the priority ordering of an organization's branding policies.

        [API documentation: updateOrganizationBrandingPoliciesPriorities](https://developer.cisco.com/meraki/api-v1/#!update-organization-branding-policies-priorities)

        Args:
            organization_id: Organization ID.
            branding_policy_ids: An ordered list of branding policy IDs that determines the priority
                order of how to apply the policies.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/brandingPolicies/priorities"

        payload: dict[str, Any] = {}
        if branding_policy_ids is not None:
            payload["brandingPolicyIds"] = branding_policy_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_organization_branding_policy(
        self,
        *,
        organization_id: str,
        branding_policy_id: str,
        name: str,
        enabled: bool | None = None,
        admin_settings: UpdateOrganizationBrandingPolicyAdminSettings | None = None,
        help_settings: UpdateOrganizationBrandingPolicyHelpSettings | None = None,
        custom_logo: UpdateOrganizationBrandingPolicyCustomLogo | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a branding policy.

        [API documentation: updateOrganizationBrandingPolicy](https://developer.cisco.com/meraki/api-v1/#!update-organization-branding-policy)

        Args:
            organization_id: Organization ID.
            branding_policy_id: Branding policy ID.
            name: Name of the Dashboard branding policy.
            enabled: Boolean indicating whether this policy is enabled.
            admin_settings: Settings for describing which kinds of admins this policy applies to.
            help_settings: Settings for describing the modifications to various Help page features.
                Each property in this object accepts one of 'default or inherit' (do not
                modify functionality), 'hide' (remove the section from Dashboard), or
                'show' (always show the section on Dashboard). Some properties in this
                object also accept custom HTML used to replace the section on Dashboard;
                see the documentation for each property to see the allowed values.
            custom_logo: Properties describing the custom logo attached to the branding policy.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        branding_policy_id = urllib.parse.quote(str(branding_policy_id), safe="")
        path = f"/organizations/{organization_id}/brandingPolicies/{branding_policy_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if enabled is not None:
            payload["enabled"] = enabled
        if admin_settings is not None:
            payload["adminSettings"] = admin_settings.model_dump(by_alias=True, exclude_none=True)
        if help_settings is not None:
            payload["helpSettings"] = help_settings.model_dump(by_alias=True, exclude_none=True)
        if custom_logo is not None:
            payload["customLogo"] = custom_logo.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_branding_policy(
        self, *, organization_id: str, branding_policy_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a branding policy.

        [API documentation: deleteOrganizationBrandingPolicy](https://developer.cisco.com/meraki/api-v1/#!delete-organization-branding-policy)

        Args:
            organization_id: Organization ID.
            branding_policy_id: Branding policy ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        branding_policy_id = urllib.parse.quote(str(branding_policy_id), safe="")
        path = f"/organizations/{organization_id}/brandingPolicies/{branding_policy_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_config_template(
        self,
        *,
        organization_id: str,
        name: str,
        time_zone: str | None = None,
        copy_from_network_id: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a new configuration template.

        [API documentation: createOrganizationConfigTemplate](https://developer.cisco.com/meraki/api-v1/#!create-organization-config-template)

        Args:
            organization_id: Organization ID.
            name: The name of the configuration template.
            time_zone: The timezone of the configuration template. For a list of allowed timezones,
                please see the 'TZ' column in the table in <a target='_blank'
                href='https://en.wikipedia.org/wiki/List_of_tz_database_time_zones'>this
                article</a>. Not applicable if copying from existing network or
                template.
            copy_from_network_id: The ID of the network or config template to copy configuration
                from.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/configTemplates"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if time_zone is not None:
            payload["timeZone"] = time_zone
        if copy_from_network_id is not None:
            payload["copyFromNetworkId"] = copy_from_network_id

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_config_template(
        self,
        *,
        organization_id: str,
        config_template_id: str,
        name: str | None = None,
        time_zone: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a configuration template.

        [API documentation: updateOrganizationConfigTemplate](https://developer.cisco.com/meraki/api-v1/#!update-organization-config-template)

        Args:
            organization_id: Organization ID.
            config_template_id: Config template ID.
            name: The name of the configuration template.
            time_zone: The timezone of the configuration template. For a list of allowed timezones,
                please see the 'TZ' column in the table in <a target='_blank'
                href='https://en.wikipedia.org/wiki/List_of_tz_database_time_zones'>this
                article.</a>.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        config_template_id = urllib.parse.quote(str(config_template_id), safe="")
        path = f"/organizations/{organization_id}/configTemplates/{config_template_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if time_zone is not None:
            payload["timeZone"] = time_zone

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_organization_devices_controller_migration(
        self,
        *,
        organization_id: str,
        serials: list[str],
        target: CreateOrganizationDevicesControllerMigrationTarget,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Migrate devices to another controller or management mode.

        [API documentation: createOrganizationDevicesControllerMigration](https://developer.cisco.com/meraki/api-v1/#!create-organization-devices-controller-migration)

        Args:
            organization_id: Organization ID.
            serials: A list of Meraki Serials to migrate.
            target: The controller or management mode to which the devices will be migrated.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/devices/controller/migrations"

        payload: dict[str, Any] = {}
        if serials is not None:
            payload["serials"] = serials
        if target is not None:
            payload["target"] = target

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="mr/actions/migrate",
            body=payload,
        )

    def bulk_update_organization_devices_details(
        self,
        *,
        organization_id: str,
        serials: list[str],
        details: list[BulkUpdateOrganizationDevicesDetailsDetailsItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updating device details (currently only used for Catalyst devices).

        [API documentation: bulkUpdateOrganizationDevicesDetails](https://developer.cisco.com/meraki/api-v1/#!bulk-update-organization-devices-details)

        Args:
            organization_id: Organization ID.
            serials: A list of serials of devices to update.
            details: An array of details.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/devices/details/bulkUpdate"

        payload: dict[str, Any] = {}
        if serials is not None:
            payload["serials"] = serials
        if details is not None:
            payload["details"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in details
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="details/update",
            body=payload,
        )

    def bulk_organization_devices_packet_capture_captures_delete(
        self, *, organization_id: str, capture_ids: list[str]
    ) -> CreateOrganizationActionBatchActionsItem:
        """BulkDelete packet captures from cloud.

        [API documentation: bulkOrganizationDevicesPacketCaptureCapturesDelete](https://developer.cisco.com/meraki/api-v1/#!bulk-organization-devices-packet-capture-captures-delete)

        Args:
            organization_id: Organization ID.
            capture_ids: Delete the packet captures of the specified capture ids.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/devices/packetCapture/captures/bulkDelete"

        payload: dict[str, Any] = {}
        if capture_ids is not None:
            payload["captureIds"] = capture_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
            body=payload,
        )

    def delete_organization_devices_packet_capture_capture(
        self, *, organization_id: str, capture_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a single packet capture from cloud using captureId.

        [API documentation: deleteOrganizationDevicesPacketCaptureCapture](https://developer.cisco.com/meraki/api-v1/#!delete-organization-devices-packet-capture-capture)

        Args:
            organization_id: Organization ID.
            capture_id: Capture ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        capture_id = urllib.parse.quote(str(capture_id), safe="")
        path = f"/organizations/{organization_id}/devices/packetCapture/captures/{capture_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_devices_packet_capture_schedule(
        self,
        *,
        organization_id: str,
        devices: list[CreateOrganizationDevicesPacketCaptureScheduleDevicesItem],
        name: str | None = None,
        notes: str | None = None,
        duration: int | None = None,
        filter_expression: str | None = None,
        enabled: bool | None = None,
        schedule: CreateOrganizationDevicesPacketCaptureScheduleSchedule | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a schedule for packet capture.

        [API documentation: createOrganizationDevicesPacketCaptureSchedule](https://developer.cisco.com/meraki/api-v1/#!create-organization-devices-packet-capture-schedule)

        Args:
            organization_id: Organization ID.
            devices: device details.
            name: Name of the packet capture file.
            notes: Reason for capture.
            duration: Duration of the capture in seconds.
            filter_expression: Filter expression for the capture.
            enabled: Enable or disable the schedule.
            schedule: Schedule details.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/devices/packetCapture/schedules"

        payload: dict[str, Any] = {}
        if devices is not None:
            payload["devices"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in devices
            ]
        if name is not None:
            payload["name"] = name
        if notes is not None:
            payload["notes"] = notes
        if duration is not None:
            payload["duration"] = duration
        if filter_expression is not None:
            payload["filterExpression"] = filter_expression
        if enabled is not None:
            payload["enabled"] = enabled
        if schedule is not None:
            payload["schedule"] = schedule.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def reorder_organization_devices_packet_capture_schedules(
        self,
        *,
        organization_id: str,
        order: list[ReorderOrganizationDevicesPacketCaptureSchedulesOrderItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Bulk update priorities of pcap schedules.

        [API documentation: reorderOrganizationDevicesPacketCaptureSchedules](https://developer.cisco.com/meraki/api-v1/#!reorder-organization-devices-packet-capture-schedules)

        Args:
            organization_id: Organization ID.
            order: Array of schedule IDs and their priorities to reorder.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/devices/packetCapture/schedules/reorder"

        payload: dict[str, Any] = {}
        if order is not None:
            payload["order"] = [item.model_dump(by_alias=True, exclude_none=True) for item in order]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="reorder",
            body=payload,
        )

    def update_organization_devices_packet_capture_schedule(
        self,
        *,
        organization_id: str,
        schedule_id: str,
        devices: list[UpdateOrganizationDevicesPacketCaptureScheduleDevicesItem],
        name: str | None = None,
        notes: str | None = None,
        duration: int | None = None,
        filter_expression: str | None = None,
        enabled: bool | None = None,
        schedule: UpdateOrganizationDevicesPacketCaptureScheduleSchedule | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a schedule for packet capture.

        [API documentation: updateOrganizationDevicesPacketCaptureSchedule](https://developer.cisco.com/meraki/api-v1/#!update-organization-devices-packet-capture-schedule)

        Args:
            organization_id: Organization ID.
            schedule_id: Schedule ID.
            devices: device details.
            name: Name of the packet capture file.
            notes: Reason for capture.
            duration: Duration of the capture in seconds.
            filter_expression: Filter expression for the capture.
            enabled: Enable or disable the schedule.
            schedule: Schedule details.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        schedule_id = urllib.parse.quote(str(schedule_id), safe="")
        path = f"/organizations/{organization_id}/devices/packetCapture/schedules/{schedule_id}"

        payload: dict[str, Any] = {}
        if devices is not None:
            payload["devices"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in devices
            ]
        if name is not None:
            payload["name"] = name
        if notes is not None:
            payload["notes"] = notes
        if duration is not None:
            payload["duration"] = duration
        if filter_expression is not None:
            payload["filterExpression"] = filter_expression
        if enabled is not None:
            payload["enabled"] = enabled
        if schedule is not None:
            payload["schedule"] = schedule.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_devices_packet_capture_schedule(
        self, *, organization_id: str, schedule_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete schedule from cloud.

        [API documentation: deleteOrganizationDevicesPacketCaptureSchedule](https://developer.cisco.com/meraki/api-v1/#!delete-organization-devices-packet-capture-schedule)

        Args:
            organization_id: Organization ID.
            schedule_id: Schedule ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        schedule_id = urllib.parse.quote(str(schedule_id), safe="")
        path = f"/organizations/{organization_id}/devices/packetCapture/schedules/{schedule_id}"

        payload: dict[str, Any] = {}
        if schedule_id is not None:
            payload["scheduleId"] = schedule_id

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
            body=payload,
        )

    def update_organization_early_access_features_opt_in(
        self,
        *,
        organization_id: str,
        opt_in_id: str,
        limit_scope_to_networks: list[str] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update an early access feature opt-in for an organization.

        [API documentation: updateOrganizationEarlyAccessFeaturesOptIn](https://developer.cisco.com/meraki/api-v1/#!update-organization-early-access-features-opt-in)

        Args:
            organization_id: Organization ID.
            opt_in_id: Opt in ID.
            limit_scope_to_networks: A list of network IDs to apply the opt-in to.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        opt_in_id = urllib.parse.quote(str(opt_in_id), safe="")
        path = f"/organizations/{organization_id}/earlyAccess/features/optIns/{opt_in_id}"

        payload: dict[str, Any] = {}
        if limit_scope_to_networks is not None:
            payload["limitScopeToNetworks"] = limit_scope_to_networks

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def disable_organization_integrations_xdr_networks(
        self,
        *,
        organization_id: str,
        networks: list[DisableOrganizationIntegrationsXdrNetworksNetworksItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Disable XDR on networks.

        [API documentation: disableOrganizationIntegrationsXdrNetworks](https://developer.cisco.com/meraki/api-v1/#!disable-organization-integrations-xdr-networks)

        Args:
            organization_id: Organization ID.
            networks: List containing the network ID and the product type to disable XDR on.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/integrations/xdr/networks/disable"

        payload: dict[str, Any] = {}
        if networks is not None:
            payload["networks"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in networks
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="disable",
            body=payload,
        )

    def enable_organization_integrations_xdr_networks(
        self,
        *,
        organization_id: str,
        networks: list[EnableOrganizationIntegrationsXdrNetworksNetworksItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Enable XDR on networks.

        [API documentation: enableOrganizationIntegrationsXdrNetworks](https://developer.cisco.com/meraki/api-v1/#!enable-organization-integrations-xdr-networks)

        Args:
            organization_id: Organization ID.
            networks: List containing the network ID and the product type to enable XDR on.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/integrations/xdr/networks/enable"

        payload: dict[str, Any] = {}
        if networks is not None:
            payload["networks"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in networks
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="enable",
            body=payload,
        )

    def claim_organization_inventory_orders(
        self,
        *,
        organization_id: str,
        claim_id: str,
        subscriptions: list[ClaimOrganizationInventoryOrdersSubscriptionsItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Claim an order by the secure unique order claim number, the order claim id.

        [API documentation: claimOrganizationInventoryOrders](https://developer.cisco.com/meraki/api-v1/#!claim-organization-inventory-orders)

        Args:
            organization_id: Organization ID.
            claim_id: The unique order claim id.
            subscriptions: The individual subscriptions to claim.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/inventory/orders/claim"

        payload: dict[str, Any] = {}
        if claim_id is not None:
            payload["claimId"] = claim_id
        if subscriptions is not None:
            payload["subscriptions"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in subscriptions
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="claim",
            body=payload,
        )

    def assign_organization_licenses_seats(
        self, *, organization_id: str, license_id: str, network_id: str, seat_count: int
    ) -> CreateOrganizationActionBatchActionsItem:
        """Assign SM seats to a network.

        [API documentation: assignOrganizationLicensesSeats](https://developer.cisco.com/meraki/api-v1/#!assign-organization-licenses-seats)

        Args:
            organization_id: Organization ID.
            license_id: The ID of the SM license to assign seats from.
            network_id: The ID of the SM network to assign the seats to.
            seat_count: The number of seats to assign to the SM network. Must be less than or equal
                to the total number of seats of the license.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/licenses/assignSeats"

        payload: dict[str, Any] = {}
        if license_id is not None:
            payload["licenseId"] = license_id
        if network_id is not None:
            payload["networkId"] = network_id
        if seat_count is not None:
            payload["seatCount"] = seat_count

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="assignSeats",
            body=payload,
        )

    def move_organization_licenses(
        self, *, organization_id: str, dest_organization_id: str, license_ids: list[str]
    ) -> CreateOrganizationActionBatchActionsItem:
        """Move licenses to another organization.

        [API documentation: moveOrganizationLicenses](https://developer.cisco.com/meraki/api-v1/#!move-organization-licenses)

        Args:
            organization_id: Organization ID.
            dest_organization_id: The ID of the organization to move the licenses to.
            license_ids: A list of IDs of licenses to move to the new organization.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/licenses/move"

        payload: dict[str, Any] = {}
        if dest_organization_id is not None:
            payload["destOrganizationId"] = dest_organization_id
        if license_ids is not None:
            payload["licenseIds"] = license_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="move",
            body=payload,
        )

    def move_organization_licenses_seats(
        self, *, organization_id: str, dest_organization_id: str, license_id: str, seat_count: int
    ) -> CreateOrganizationActionBatchActionsItem:
        """Move SM seats to another organization.

        [API documentation: moveOrganizationLicensesSeats](https://developer.cisco.com/meraki/api-v1/#!move-organization-licenses-seats)

        Args:
            organization_id: Organization ID.
            dest_organization_id: The ID of the organization to move the SM seats to.
            license_id: The ID of the SM license to move the seats from.
            seat_count: The number of seats to move to the new organization. Must be less than or
                equal to the total number of seats of the license.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/licenses/moveSeats"

        payload: dict[str, Any] = {}
        if dest_organization_id is not None:
            payload["destOrganizationId"] = dest_organization_id
        if license_id is not None:
            payload["licenseId"] = license_id
        if seat_count is not None:
            payload["seatCount"] = seat_count

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="moveSeats",
            body=payload,
        )

    def renew_organization_licenses_seats(
        self, *, organization_id: str, license_id_to_renew: str, unused_license_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Renew SM seats of a license.

        [API documentation: renewOrganizationLicensesSeats](https://developer.cisco.com/meraki/api-v1/#!renew-organization-licenses-seats)

        Args:
            organization_id: Organization ID.
            license_id_to_renew: The ID of the SM license to renew. This license must already be
                assigned to an SM network.
            unused_license_id: The SM license to use to renew the seats on 'licenseIdToRenew'. This
                license must have at least as many seats available as there are seats on
                'licenseIdToRenew'.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/licenses/renewSeats"

        payload: dict[str, Any] = {}
        if license_id_to_renew is not None:
            payload["licenseIdToRenew"] = license_id_to_renew
        if unused_license_id is not None:
            payload["unusedLicenseId"] = unused_license_id

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="renewSeats",
            body=payload,
        )

    def update_organization_license(
        self, *, organization_id: str, license_id: str, device_serial: str | None = None
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a license.

        [API documentation: updateOrganizationLicense](https://developer.cisco.com/meraki/api-v1/#!update-organization-license)

        Args:
            organization_id: Organization ID.
            license_id: License ID.
            device_serial: The serial number of the device to assign this license to. Set this to
                null to unassign the license. If a different license is already active
                on the device, this parameter will control queueing/dequeuing this
                license.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        license_id = urllib.parse.quote(str(license_id), safe="")
        path = f"/organizations/{organization_id}/licenses/{license_id}"

        payload: dict[str, Any] = {}
        if device_serial is not None:
            payload["deviceSerial"] = device_serial

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_organization_login_security(
        self,
        organization_id: str,
        *,
        enforce_password_expiration: bool | None = None,
        password_expiration_days: int | None = None,
        enforce_different_passwords: bool | None = None,
        num_different_passwords: int | None = None,
        enforce_strong_passwords: bool | None = None,
        minimum_password_length: int | None = None,
        enforce_account_lockout: bool | None = None,
        account_lockout_attempts: int | None = None,
        enforce_idle_timeout: bool | None = None,
        idle_timeout_minutes: int | None = None,
        enforce_two_factor_auth: bool | None = None,
        enforce_login_ip_ranges: bool | None = None,
        login_ip_ranges: list[str] | None = None,
        api_authentication: UpdateOrganizationLoginSecurityApiAuthentication | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the login security settings for an organization.

        [API documentation: updateOrganizationLoginSecurity](https://developer.cisco.com/meraki/api-v1/#!update-organization-login-security)

        Args:
            organization_id: Organization ID.
            enforce_password_expiration: Boolean indicating whether users are forced to change their
                password every X number of days.
            password_expiration_days: Number of days after which users will be forced to change
                their password.
            enforce_different_passwords: Boolean indicating whether users, when setting a new
                password, are forced to choose a new password that is different from any
                past passwords.
            num_different_passwords: Number of recent passwords that new password must be distinct
                from.
            enforce_strong_passwords: Deprecated. Values of 'false' are always ignored.
            minimum_password_length: Minimum number of characters required in admins' passwords.
            enforce_account_lockout: Boolean indicating whether users' Dashboard accounts will be
                locked out after a specified number of consecutive failed login
                attempts.
            account_lockout_attempts: Number of consecutive failed login attempts after which users'
                accounts will be locked.
            enforce_idle_timeout: Boolean indicating whether users will be logged out after being
                idle for the specified number of minutes.
            idle_timeout_minutes: Number of minutes users can remain idle before being logged out of
                their accounts.
            enforce_two_factor_auth: Boolean indicating whether users in this organization will be
                required to use an extra verification code when logging in to Dashboard.
                This code will be sent to their mobile phone via SMS, or can be
                generated by the authenticator application.
            enforce_login_ip_ranges: Boolean indicating whether organization will restrict access to
                Dashboard (including the API) from certain IP addresses.
            login_ip_ranges: List of acceptable IP ranges. Entries can be single IP addresses, IP
                address ranges, and CIDR subnets.
            api_authentication: Details for indicating whether organization will restrict access to
                API (but not Dashboard) to certain IP addresses.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/loginSecurity"

        payload: dict[str, Any] = {}
        if enforce_password_expiration is not None:
            payload["enforcePasswordExpiration"] = enforce_password_expiration
        if password_expiration_days is not None:
            payload["passwordExpirationDays"] = password_expiration_days
        if enforce_different_passwords is not None:
            payload["enforceDifferentPasswords"] = enforce_different_passwords
        if num_different_passwords is not None:
            payload["numDifferentPasswords"] = num_different_passwords
        if enforce_strong_passwords is not None:
            payload["enforceStrongPasswords"] = enforce_strong_passwords
        if minimum_password_length is not None:
            payload["minimumPasswordLength"] = minimum_password_length
        if enforce_account_lockout is not None:
            payload["enforceAccountLockout"] = enforce_account_lockout
        if account_lockout_attempts is not None:
            payload["accountLockoutAttempts"] = account_lockout_attempts
        if enforce_idle_timeout is not None:
            payload["enforceIdleTimeout"] = enforce_idle_timeout
        if idle_timeout_minutes is not None:
            payload["idleTimeoutMinutes"] = idle_timeout_minutes
        if enforce_two_factor_auth is not None:
            payload["enforceTwoFactorAuth"] = enforce_two_factor_auth
        if enforce_login_ip_ranges is not None:
            payload["enforceLoginIpRanges"] = enforce_login_ip_ranges
        if login_ip_ranges is not None:
            payload["loginIpRanges"] = login_ip_ranges
        if api_authentication is not None:
            payload["apiAuthentication"] = api_authentication.model_dump(
                by_alias=True, exclude_none=True
            )

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_organization_network(
        self,
        *,
        organization_id: str,
        name: str,
        product_types: CreateOrganizationNetworkProductTypes,
        tags: list[str] | None = None,
        time_zone: str | None = None,
        copy_from_network_id: str | None = None,
        notes: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a network.

        [API documentation: createOrganizationNetwork](https://developer.cisco.com/meraki/api-v1/#!create-organization-network)

        Args:
            organization_id: Organization ID.
            name: The name of the new network.
            product_types: The product type(s) of the new network. If more than one type is
                included, the network will be a combined network.
            tags: A list of tags to be applied to the network.
            time_zone: The timezone of the network. For a list of allowed timezones, please see the
                'TZ' column in the table in <a target='_blank'
                href='https://en.wikipedia.org/wiki/List_of_tz_database_time_zones'>this
                article.</a>.
            copy_from_network_id: The ID of the network to copy configuration from. Other provided
                parameters will override the copied configuration, except type which
                must match this network's type exactly.
            notes: Add any notes or additional information about this network here.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/networks"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if product_types is not None:
            payload["productTypes"] = product_types
        if tags is not None:
            payload["tags"] = tags
        if time_zone is not None:
            payload["timeZone"] = time_zone
        if copy_from_network_id is not None:
            payload["copyFromNetworkId"] = copy_from_network_id
        if notes is not None:
            payload["notes"] = notes

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def combine_organization_networks(
        self,
        *,
        organization_id: str,
        name: str,
        network_ids: list[str],
        enrollment_string: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Combine multiple networks into a single network.

        [API documentation: combineOrganizationNetworks](https://developer.cisco.com/meraki/api-v1/#!combine-organization-networks)

        Args:
            organization_id: Organization ID.
            name: The name of the combined network.
            network_ids: A list of the network IDs that will be combined. If an ID of a combined
                network is included in this list, the other networks in the list will be
                grouped into that network.
            enrollment_string: A unique identifier which can be used for device enrollment or easy
                access through the Meraki SM Registration page or the Self Service
                Portal. Please note that changing this field may cause existing
                bookmarks to break. All networks that are part of this combined network
                will have their enrollment string appended by '-network_type'. If left
                empty, all exisitng enrollment strings will be deleted.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/networks/combine"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if network_ids is not None:
            payload["networkIds"] = network_ids
        if enrollment_string is not None:
            payload["enrollmentString"] = enrollment_string

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="combine",
            body=payload,
        )

    def create_organization_policy_object(
        self,
        *,
        organization_id: str,
        name: str,
        category: str,
        type_: str,
        cidr: str | None = None,
        fqdn: str | None = None,
        mask: str | None = None,
        ip: str | None = None,
        group_ids: list[str] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Creates a new Policy Object.

        [API documentation: createOrganizationPolicyObject](https://developer.cisco.com/meraki/api-v1/#!create-organization-policy-object)

        Args:
            organization_id: Organization ID.
            name: Name of a policy object, unique within the organization (alphanumeric, space,
                dash, or underscore characters only).
            category: Category of a policy object (one of: adaptivePolicy, network).
            type_: Type of a policy object (one of: adaptivePolicyIpv4Cidr, cidr, fqdn, ipAndMask).
            cidr: CIDR Value of a policy object (e.g. 10.11.12.1/24").
            fqdn: Fully qualified domain name of policy object (e.g. "example.com").
            mask: Mask of a policy object (e.g. "255.255.0.0").
            ip: IP Address of a policy object (e.g. "1.2.3.4").
            group_ids: The IDs of policy object groups the policy object belongs to.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/policyObjects"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if category is not None:
            payload["category"] = category
        if type_ is not None:
            payload["type"] = type_
        if cidr is not None:
            payload["cidr"] = cidr
        if fqdn is not None:
            payload["fqdn"] = fqdn
        if mask is not None:
            payload["mask"] = mask
        if ip is not None:
            payload["ip"] = ip
        if group_ids is not None:
            payload["groupIds"] = group_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def create_organization_policy_objects_group(
        self,
        *,
        organization_id: str,
        name: str,
        category: str | None = None,
        object_ids: list[str] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Creates a new Policy Object Group.

        [API documentation: createOrganizationPolicyObjectsGroup](https://developer.cisco.com/meraki/api-v1/#!create-organization-policy-objects-group)

        Args:
            organization_id: Organization ID.
            name: A name for the group of network addresses, unique within the organization
                (alphanumeric, space, dash, or underscore characters only).
            category: Category of a policy object group (one of: NetworkObjectGroup,
                GeoLocationGroup, PortObjectGroup, ApplicationGroup).
            object_ids: A list of Policy Object ID's that this NetworkObjectGroup should be
                associated to (note: these ID's will replace the existing associated
                Policy Objects).

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/policyObjects/groups"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if category is not None:
            payload["category"] = category
        if object_ids is not None:
            payload["objectIds"] = object_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_policy_objects_group(
        self,
        *,
        organization_id: str,
        policy_object_group_id: str,
        name: str | None = None,
        object_ids: list[str] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates a Policy Object Group.

        [API documentation: updateOrganizationPolicyObjectsGroup](https://developer.cisco.com/meraki/api-v1/#!update-organization-policy-objects-group)

        Args:
            organization_id: Organization ID.
            policy_object_group_id: Policy object group ID.
            name: A name for the group of network addresses, unique within the organization
                (alphanumeric, space, dash, or underscore characters only).
            object_ids: A list of Policy Object ID's that this NetworkObjectGroup should be
                associated to (note: these ID's will replace the existing associated
                Policy Objects).

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        policy_object_group_id = urllib.parse.quote(str(policy_object_group_id), safe="")
        path = f"/organizations/{organization_id}/policyObjects/groups/{policy_object_group_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if object_ids is not None:
            payload["objectIds"] = object_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_policy_objects_group(
        self, *, organization_id: str, policy_object_group_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes a Policy Object Group.

        [API documentation: deleteOrganizationPolicyObjectsGroup](https://developer.cisco.com/meraki/api-v1/#!delete-organization-policy-objects-group)

        Args:
            organization_id: Organization ID.
            policy_object_group_id: Policy object group ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        policy_object_group_id = urllib.parse.quote(str(policy_object_group_id), safe="")
        path = f"/organizations/{organization_id}/policyObjects/groups/{policy_object_group_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def update_organization_policy_object(
        self,
        *,
        organization_id: str,
        policy_object_id: str,
        name: str | None = None,
        cidr: str | None = None,
        fqdn: str | None = None,
        mask: str | None = None,
        ip: str | None = None,
        group_ids: list[str] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates a Policy Object.

        [API documentation: updateOrganizationPolicyObject](https://developer.cisco.com/meraki/api-v1/#!update-organization-policy-object)

        Args:
            organization_id: Organization ID.
            policy_object_id: Policy object ID.
            name: Name of a policy object, unique within the organization (alphanumeric, space,
                dash, or underscore characters only).
            cidr: CIDR Value of a policy object (e.g. 10.11.12.1/24").
            fqdn: Fully qualified domain name of policy object (e.g. "example.com").
            mask: Mask of a policy object (e.g. "255.255.0.0").
            ip: IP Address of a policy object (e.g. "1.2.3.4").
            group_ids: The IDs of policy object groups the policy object belongs to.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        policy_object_id = urllib.parse.quote(str(policy_object_id), safe="")
        path = f"/organizations/{organization_id}/policyObjects/{policy_object_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if cidr is not None:
            payload["cidr"] = cidr
        if fqdn is not None:
            payload["fqdn"] = fqdn
        if mask is not None:
            payload["mask"] = mask
        if ip is not None:
            payload["ip"] = ip
        if group_ids is not None:
            payload["groupIds"] = group_ids

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_policy_object(
        self, *, organization_id: str, policy_object_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes a Policy Object.

        [API documentation: deleteOrganizationPolicyObject](https://developer.cisco.com/meraki/api-v1/#!delete-organization-policy-object)

        Args:
            organization_id: Organization ID.
            policy_object_id: Policy object ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        policy_object_id = urllib.parse.quote(str(policy_object_id), safe="")
        path = f"/organizations/{organization_id}/policyObjects/{policy_object_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_saml_idp(
        self,
        *,
        organization_id: str,
        x509cert_sha1_fingerprint: str,
        sso_login_url: str | None = None,
        slo_logout_url: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a SAML IdP for your organization.

        [API documentation: createOrganizationSamlIdp](https://developer.cisco.com/meraki/api-v1/#!create-organization-saml-idp)

        Args:
            organization_id: Organization ID.
            x509cert_sha1_fingerprint: Fingerprint (SHA1) of the SAML certificate provided by your
                Identity Provider (IdP). This will be used for encryption / validation.
            sso_login_url: Dashboard will redirect users to this URL to log in again when their
                sessions expire.
            slo_logout_url: Dashboard will redirect users to this URL when they sign out.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/saml/idps"

        payload: dict[str, Any] = {}
        if x509cert_sha1_fingerprint is not None:
            payload["x509certSha1Fingerprint"] = x509cert_sha1_fingerprint
        if sso_login_url is not None:
            payload["ssoLoginUrl"] = sso_login_url
        if slo_logout_url is not None:
            payload["sloLogoutUrl"] = slo_logout_url

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_saml_idp(
        self,
        *,
        organization_id: str,
        idp_id: str,
        x509cert_sha1_fingerprint: str | None = None,
        sso_login_url: str | None = None,
        slo_logout_url: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a SAML IdP in your organization.

        [API documentation: updateOrganizationSamlIdp](https://developer.cisco.com/meraki/api-v1/#!update-organization-saml-idp)

        Args:
            organization_id: Organization ID.
            idp_id: Idp ID.
            x509cert_sha1_fingerprint: Fingerprint (SHA1) of the SAML certificate provided by your
                Identity Provider (IdP). This will be used for encryption / validation.
            sso_login_url: Dashboard will redirect users to this URL to log in again when their
                sessions expire.
            slo_logout_url: Dashboard will redirect users to this URL when they sign out.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        idp_id = urllib.parse.quote(str(idp_id), safe="")
        path = f"/organizations/{organization_id}/saml/idps/{idp_id}"

        payload: dict[str, Any] = {}
        if x509cert_sha1_fingerprint is not None:
            payload["x509certSha1Fingerprint"] = x509cert_sha1_fingerprint
        if sso_login_url is not None:
            payload["ssoLoginUrl"] = sso_login_url
        if slo_logout_url is not None:
            payload["sloLogoutUrl"] = slo_logout_url

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_saml_idp(
        self, *, organization_id: str, idp_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Remove a SAML IdP in your organization.

        [API documentation: deleteOrganizationSamlIdp](https://developer.cisco.com/meraki/api-v1/#!delete-organization-saml-idp)

        Args:
            organization_id: Organization ID.
            idp_id: Idp ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        idp_id = urllib.parse.quote(str(idp_id), safe="")
        path = f"/organizations/{organization_id}/saml/idps/{idp_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def delete_organization_splash_asset(
        self, *, organization_id: str, id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a Splash Theme Asset.

        [API documentation: deleteOrganizationSplashAsset](https://developer.cisco.com/meraki/api-v1/#!delete-organization-splash-asset)

        Args:
            organization_id: Organization ID.
            id: ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/splash/assets/{id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_splash_theme(
        self, organization_id: str, *, name: str | None = None, base_theme: str | None = None
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a Splash Theme.

        [API documentation: createOrganizationSplashTheme](https://developer.cisco.com/meraki/api-v1/#!create-organization-splash-theme)

        Args:
            organization_id: Organization ID.
            name: theme name.
            base_theme: base theme id.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/splash/themes"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if base_theme is not None:
            payload["baseTheme"] = base_theme

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def delete_organization_splash_theme(
        self, *, organization_id: str, id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a Splash Theme.

        [API documentation: deleteOrganizationSplashTheme](https://developer.cisco.com/meraki/api-v1/#!delete-organization-splash-theme)

        Args:
            organization_id: Organization ID.
            id: ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/splash/themes/{id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_splash_theme_asset(
        self,
        *,
        organization_id: str,
        theme_identifier: str,
        name: str | None = None,
        content: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a Splash Theme Asset.

        [API documentation: createOrganizationSplashThemeAsset](https://developer.cisco.com/meraki/api-v1/#!create-organization-splash-theme-asset)

        Args:
            organization_id: Organization ID.
            theme_identifier: Theme identifier.
            name: File name. Will overwrite files with same name.
            content: a file containing the asset content.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        theme_identifier = urllib.parse.quote(str(theme_identifier), safe="")
        path = f"/organizations/{organization_id}/splash/themes/{theme_identifier}/assets"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if content is not None:
            payload["content"] = content

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )



"""CellularGateway API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from __future__ import annotations

import urllib.parse
from typing import TYPE_CHECKING, Any, Literal

from meraki_client.schemas import (
    CreateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse,
    CreateOrganizationCellularGatewayEsimsServiceProvidersAccountServiceProvider,
    CreateOrganizationCellularGatewayEsimsSwapResponse,
    CreateOrganizationCellularGatewayEsimsSwapSwapsItem,
    GetDeviceCellularGatewayLanResponse,
    GetDeviceCellularGatewayPortForwardingRulesResponse,
    GetNetworkCellularGatewayConnectivityMonitoringDestinationsResponse,
    GetNetworkCellularGatewayDhcpResponse,
    GetNetworkCellularGatewaySubnetPoolResponse,
    GetNetworkCellularGatewayUplinkResponse,
    GetOrganizationCellularGatewayEsimsInventoryResponseItemsItem,
    GetOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlansResponseItemsItem,
    GetOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlansResponseItemsItem,
    GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponseItem,
    GetOrganizationCellularGatewayEsimsServiceProvidersResponseItemsItem,
    GetOrganizationCellularGatewayUplinkStatusesResponseItem,
    UpdateDeviceCellularGatewayLanFixedIpAssignmentsItem,
    UpdateDeviceCellularGatewayLanReservedIpRangesItem,
    UpdateDeviceCellularGatewayLanResponse,
    UpdateDeviceCellularGatewayPortForwardingRulesResponse,
    UpdateDeviceCellularGatewayPortForwardingRulesRulesItem,
    UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsDestinationsItem,
    UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsResponse,
    UpdateNetworkCellularGatewayDhcpResponse,
    UpdateNetworkCellularGatewaySubnetPoolResponse,
    UpdateNetworkCellularGatewayUplinkBandwidthLimits,
    UpdateNetworkCellularGatewayUplinkResponse,
    UpdateOrganizationCellularGatewayEsimsInventoryResponse,
    UpdateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse,
    UpdateOrganizationCellularGatewayEsimsSwapResponse,
)

if TYPE_CHECKING:
    from meraki_client._session import PaginatedResponse, Session


class CellularGateway:
    """CellularGateway class."""

    def __init__(self, session: Session) -> None:
        self._session = session

    def get_device_cellular_gateway_lan(self, serial: str) -> GetDeviceCellularGatewayLanResponse:
        """Show the LAN Settings of a MG.

        [API documentation: getDeviceCellularGatewayLan](https://developer.cisco.com/meraki/api-v1/#!get-device-cellular-gateway-lan)

        Args:
            serial: Serial.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "deviceName": "name of the MG",
              "deviceLanIp": "192.168.0.33",
              "deviceSubnet": "192.168.0.32/27",
              "fixedIpAssignments": [
                {
                  "name": "server 1",
                  "ip": "192.168.0.10",
                  "mac": "0b:00:00:00:00:ac"
                }
              ],
              "reservedIpRanges": [
                {
                  "start": "192.168.1.0",
                  "end": "192.168.1.1",
                  "comment": "A reserved IP range"
                }
              ]
            }
            ```

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/cellularGateway/lan"

        return self._session.get(
            scope="cellularGateway",
            operation_id="getDeviceCellularGatewayLan",
            path=path,
            response_schema=GetDeviceCellularGatewayLanResponse,
        )

    def update_device_cellular_gateway_lan(
        self,
        serial: str,
        *,
        reserved_ip_ranges: list[UpdateDeviceCellularGatewayLanReservedIpRangesItem] | None = None,
        fixed_ip_assignments: list[UpdateDeviceCellularGatewayLanFixedIpAssignmentsItem]
        | None = None,
    ) -> UpdateDeviceCellularGatewayLanResponse:
        """Update the LAN Settings for a single MG.

        [API documentation: updateDeviceCellularGatewayLan](https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-gateway-lan)

        Args:
            serial: Serial.
            reserved_ip_ranges: list of all reserved IP ranges for a single MG.
            fixed_ip_assignments: list of all fixed IP assignments for a single MG.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "deviceName": "name of the MG",
              "deviceLanIp": "192.168.0.33",
              "deviceSubnet": "192.168.0.32/27",
              "fixedIpAssignments": [
                {
                  "name": "server 1",
                  "ip": "192.168.0.10",
                  "mac": "0b:00:00:00:00:ac"
                }
              ],
              "reservedIpRanges": [
                {
                  "start": "192.168.1.0",
                  "end": "192.168.1.1",
                  "comment": "A reserved IP range"
                }
              ]
            }
            ```

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/cellularGateway/lan"

        payload: dict[str, Any] = {}
        if reserved_ip_ranges is not None:
            payload["reservedIpRanges"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in reserved_ip_ranges
            ]
        if fixed_ip_assignments is not None:
            payload["fixedIpAssignments"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in fixed_ip_assignments
            ]

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateDeviceCellularGatewayLan",
            path=path,
            json=payload,
            response_schema=UpdateDeviceCellularGatewayLanResponse,
        )

    def get_device_cellular_gateway_port_forwarding_rules(
        self, serial: str
    ) -> GetDeviceCellularGatewayPortForwardingRulesResponse:
        """Returns the port forwarding rules for a single MG.

        [API documentation: getDeviceCellularGatewayPortForwardingRules](https://developer.cisco.com/meraki/api-v1/#!get-device-cellular-gateway-port-forwarding-rules)

        Args:
            serial: Serial.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "rules": [
                {
                  "name": "test",
                  "lanIp": "172.31.128.5",
                  "publicPort": "11-12",
                  "localPort": "4",
                  "allowedIps": [
                    "10.10.10.10",
                    "10.10.10.11"
                  ],
                  "protocol": "tcp",
                  "access": "any"
                }
              ]
            }
            ```

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/cellularGateway/portForwardingRules"

        return self._session.get(
            scope="cellularGateway",
            operation_id="getDeviceCellularGatewayPortForwardingRules",
            path=path,
            response_schema=GetDeviceCellularGatewayPortForwardingRulesResponse,
        )

    def update_device_cellular_gateway_port_forwarding_rules(
        self,
        serial: str,
        *,
        rules: list[UpdateDeviceCellularGatewayPortForwardingRulesRulesItem] | None = None,
    ) -> UpdateDeviceCellularGatewayPortForwardingRulesResponse:
        """Updates the port forwarding rules for a single MG.

        [API documentation: updateDeviceCellularGatewayPortForwardingRules](https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-gateway-port-forwarding-rules)

        Args:
            serial: Serial.
            rules: An array of port forwarding params.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "rules": [
                {
                  "name": "test",
                  "lanIp": "172.31.128.5",
                  "publicPort": "11-12",
                  "localPort": "4",
                  "allowedIps": [
                    "10.10.10.10",
                    "10.10.10.11"
                  ],
                  "protocol": "tcp",
                  "access": "any"
                }
              ]
            }
            ```

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/cellularGateway/portForwardingRules"

        payload: dict[str, Any] = {}
        if rules is not None:
            payload["rules"] = [item.model_dump(by_alias=True, exclude_none=True) for item in rules]

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateDeviceCellularGatewayPortForwardingRules",
            path=path,
            json=payload,
            response_schema=UpdateDeviceCellularGatewayPortForwardingRulesResponse,
        )

    def get_network_cellular_gateway_connectivity_monitoring_destinations(
        self, network_id: str
    ) -> GetNetworkCellularGatewayConnectivityMonitoringDestinationsResponse:
        """Return the connectivity testing destinations for an MG network.

        [API documentation: getNetworkCellularGatewayConnectivityMonitoringDestinations](https://developer.cisco.com/meraki/api-v1/#!get-network-cellular-gateway-connectivity-monitoring-destinations)

        Args:
            network_id: Network ID.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "destinations": [
                {
                  "ip": "1.2.3.4",
                  "description": "Google",
                  "default": false
                }
              ]
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/connectivityMonitoringDestinations"

        return self._session.get(
            scope="cellularGateway",
            operation_id="getNetworkCellularGatewayConnectivityMonitoringDestinations",
            path=path,
            response_schema=GetNetworkCellularGatewayConnectivityMonitoringDestinationsResponse,
        )

    def update_network_cellular_gateway_connectivity_monitoring_destinations(
        self,
        network_id: str,
        *,
        destinations: list[
            UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsDestinationsItem
        ]
        | None = None,
    ) -> UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsResponse:
        """Update the connectivity testing destinations for an MG network.

        [API documentation: updateNetworkCellularGatewayConnectivityMonitoringDestinations](https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-connectivity-monitoring-destinations)

        Args:
            network_id: Network ID.
            destinations: The list of connectivity monitoring destinations.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "destinations": [
                {
                  "ip": "1.2.3.4",
                  "description": "Google",
                  "default": false
                }
              ]
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/connectivityMonitoringDestinations"

        payload: dict[str, Any] = {}
        if destinations is not None:
            payload["destinations"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in destinations
            ]

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateNetworkCellularGatewayConnectivityMonitoringDestinations",
            path=path,
            json=payload,
            response_schema=UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsResponse,
        )

    def get_network_cellular_gateway_dhcp(
        self, network_id: str
    ) -> GetNetworkCellularGatewayDhcpResponse:
        """List common DHCP settings of MGs.

        [API documentation: getNetworkCellularGatewayDhcp](https://developer.cisco.com/meraki/api-v1/#!get-network-cellular-gateway-dhcp)

        Args:
            network_id: Network ID.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "dhcpLeaseTime": "1 hour",
              "dnsNameservers": "custom",
              "dnsCustomNameservers": [
                "172.16.2.111",
                "172.16.2.30"
              ]
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/dhcp"

        return self._session.get(
            scope="cellularGateway",
            operation_id="getNetworkCellularGatewayDhcp",
            path=path,
            response_schema=GetNetworkCellularGatewayDhcpResponse,
        )

    def update_network_cellular_gateway_dhcp(
        self,
        network_id: str,
        *,
        dhcp_lease_time: str | None = None,
        dns_nameservers: str | None = None,
        dns_custom_nameservers: list[str] | None = None,
    ) -> UpdateNetworkCellularGatewayDhcpResponse:
        """Update common DHCP settings of MGs.

        [API documentation: updateNetworkCellularGatewayDhcp](https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-dhcp)

        Args:
            network_id: Network ID.
            dhcp_lease_time: DHCP Lease time for all MG of the network. Possible values are '30
                minutes', '1 hour', '4 hours', '12 hours', '1 day' or '1 week'.
            dns_nameservers: DNS name servers mode for all MG of the network. Possible values are:
                'upstream_dns', 'google_dns', 'opendns', 'custom'.
            dns_custom_nameservers: list of fixed IPs representing the the DNS Name servers when the
                mode is 'custom'.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "dhcpLeaseTime": "1 hour",
              "dnsNameservers": "custom",
              "dnsCustomNameservers": [
                "172.16.2.111",
                "172.16.2.30"
              ]
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/dhcp"

        payload: dict[str, Any] = {}
        if dhcp_lease_time is not None:
            payload["dhcpLeaseTime"] = dhcp_lease_time
        if dns_nameservers is not None:
            payload["dnsNameservers"] = dns_nameservers
        if dns_custom_nameservers is not None:
            payload["dnsCustomNameservers"] = dns_custom_nameservers

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateNetworkCellularGatewayDhcp",
            path=path,
            json=payload,
            response_schema=UpdateNetworkCellularGatewayDhcpResponse,
        )

    def get_network_cellular_gateway_subnet_pool(
        self, network_id: str
    ) -> GetNetworkCellularGatewaySubnetPoolResponse:
        """Return the subnet pool and mask configured for MGs in the network.

        [API documentation: getNetworkCellularGatewaySubnetPool](https://developer.cisco.com/meraki/api-v1/#!get-network-cellular-gateway-subnet-pool)

        Args:
            network_id: Network ID.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "deploymentMode": "routed",
              "cidr": "192.168.0.0/16",
              "mask": 24,
              "subnets": [
                {
                  "serial": "AAAA-AAAA-AAAA",
                  "name": "my first MG",
                  "applianceIp": "192.168.0.1",
                  "subnet": "192.168.0.0/24"
                }
              ]
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/subnetPool"

        return self._session.get(
            scope="cellularGateway",
            operation_id="getNetworkCellularGatewaySubnetPool",
            path=path,
            response_schema=GetNetworkCellularGatewaySubnetPoolResponse,
        )

    def update_network_cellular_gateway_subnet_pool(
        self, network_id: str, *, mask: int | None = None, cidr: str | None = None
    ) -> UpdateNetworkCellularGatewaySubnetPoolResponse:
        """Update the subnet pool and mask configuration for MGs in the network.

        [API documentation: updateNetworkCellularGatewaySubnetPool](https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-subnet-pool)

        Args:
            network_id: Network ID.
            mask: Mask used for the subnet of all MGs in this network.
            cidr: CIDR of the pool of subnets. Each MG in this network will automatically pick a
                subnet from this pool.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "deploymentMode": "routed",
              "cidr": "192.168.0.0/16",
              "mask": 24,
              "subnets": [
                {
                  "serial": "AAAA-AAAA-AAAA",
                  "name": "my first MG",
                  "applianceIp": "192.168.0.1",
                  "subnet": "192.168.0.0/24"
                }
              ]
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/subnetPool"

        payload: dict[str, Any] = {}
        if mask is not None:
            payload["mask"] = mask
        if cidr is not None:
            payload["cidr"] = cidr

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateNetworkCellularGatewaySubnetPool",
            path=path,
            json=payload,
            response_schema=UpdateNetworkCellularGatewaySubnetPoolResponse,
        )

    def get_network_cellular_gateway_uplink(
        self, network_id: str
    ) -> GetNetworkCellularGatewayUplinkResponse:
        """Returns the uplink settings for your MG network.

        [API documentation: getNetworkCellularGatewayUplink](https://developer.cisco.com/meraki/api-v1/#!get-network-cellular-gateway-uplink)

        Args:
            network_id: Network ID.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "bandwidthLimits": {
                "limitUp": 51200,
                "limitDown": 51200
              }
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/uplink"

        return self._session.get(
            scope="cellularGateway",
            operation_id="getNetworkCellularGatewayUplink",
            path=path,
            response_schema=GetNetworkCellularGatewayUplinkResponse,
        )

    def update_network_cellular_gateway_uplink(
        self,
        network_id: str,
        *,
        bandwidth_limits: UpdateNetworkCellularGatewayUplinkBandwidthLimits | None = None,
    ) -> UpdateNetworkCellularGatewayUplinkResponse:
        """Updates the uplink settings for your MG network.

        [API documentation: updateNetworkCellularGatewayUplink](https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-uplink)

        Args:
            network_id: Network ID.
            bandwidth_limits: The bandwidth settings for the 'cellular' uplink.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "bandwidthLimits": {
                "limitUp": 51200,
                "limitDown": 51200
              }
            }
            ```

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/cellularGateway/uplink"

        payload: dict[str, Any] = {}
        if bandwidth_limits is not None:
            payload["bandwidthLimits"] = bandwidth_limits.model_dump(
                by_alias=True, exclude_none=True
            )

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateNetworkCellularGatewayUplink",
            path=path,
            json=payload,
            response_schema=UpdateNetworkCellularGatewayUplinkResponse,
        )

    def get_organization_cellular_gateway_esims_inventory(
        self, organization_id: str, *, eids: list[str] | None = None
    ) -> PaginatedResponse[GetOrganizationCellularGatewayEsimsInventoryResponseItemsItem]:
        """The eSIM inventory of a given organization.

        [API documentation: getOrganizationCellularGatewayEsimsInventory](https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-esims-inventory)

        Args:
            organization_id: Organization ID.
            eids: Optional parameter to filter the results by EID.

        Returns:
            Successful operation.

        Note:
            Returns a lazy PaginatedResponse
            that can be iterated or collected with `.collect()`.
            Page metadata is available on `.meta` and `.meta_pages`.

        Example API response:
            ```json
            {
              "items": [
                {
                  "device": {
                    "name": "My cellular gateway",
                    "model": "mg52",
                    "serial": "Q234-ABCD-5678",
                    "url": "https://n1.meraki.com//n//manage/nodes/new_list/000000000000",
                    "status": "online"
                  },
                  "active": true,
                  "eid": "89000000000000000000000000000000",
                  "lastUpdatedAt": "2023-02-01T00:00:00Z",
                  "network": {
                    "id": "N_24329156"
                  },
                  "profiles": [
                    {
                      "customApns": [
                        "internet"
                      ],
                      "iccid": "8900000000000000000",
                      "status": "activated",
                      "serviceProvider": {
                        "name": "ATT",
                        "plans": [
                          {
                            "name": "1 Cisco IoT SDO AT&T eSIM Test Plan downloadable",
                            "type": "communication"
                          }
                        ]
                      }
                    }
                  ]
                }
              ],
              "meta": {
                "counts": {
                  "items": {
                    "total": 1,
                    "remaining": 0
                  }
                }
              }
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/inventory"

        params: dict[str, Any] = {}
        if eids is not None:
            params["eids[]"] = eids

        return self._session.get_pages(
            scope="cellularGateway",
            operation_id="getOrganizationCellularGatewayEsimsInventory",
            path=path,
            params=params,
            item_schema=GetOrganizationCellularGatewayEsimsInventoryResponseItemsItem,
        )

    def update_organization_cellular_gateway_esims_inventory(
        self, *, organization_id: str, id: str, status: str | None = None
    ) -> UpdateOrganizationCellularGatewayEsimsInventoryResponse:
        """Toggle the status of an eSIM.

        [API documentation: updateOrganizationCellularGatewayEsimsInventory](https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-inventory)

        Args:
            organization_id: Organization ID.
            id: ID.
            status: Status the eSIM will be updated to.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "device": {
                "name": "My cellular gateway",
                "model": "mg52",
                "serial": "Q234-ABCD-5678",
                "url": "https://n1.meraki.com//n//manage/nodes/new_list/000000000000",
                "status": "online"
              },
              "active": true,
              "eid": "89000000000000000000000000000000",
              "lastUpdatedAt": "2023-02-01T00:00:00Z",
              "network": {
                "id": "N_24329156"
              },
              "profiles": [
                {
                  "customApns": [
                    "internet"
                  ],
                  "iccid": "8900000000000000000",
                  "status": "activated",
                  "serviceProvider": {
                    "name": "ATT",
                    "plans": [
                      {
                        "name": "1 Cisco IoT SDO AT&T eSIM Test Plan downloadable",
                        "type": "communication"
                      }
                    ]
                  }
                }
              ]
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        id = urllib.parse.quote(str(id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/inventory/{id}"

        payload: dict[str, Any] = {}
        if status is not None:
            payload["status"] = status

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateOrganizationCellularGatewayEsimsInventory",
            path=path,
            json=payload,
            response_schema=UpdateOrganizationCellularGatewayEsimsInventoryResponse,
        )

    def get_organization_cellular_gateway_esims_service_providers(
        self, organization_id: str
    ) -> PaginatedResponse[GetOrganizationCellularGatewayEsimsServiceProvidersResponseItemsItem]:
        """Service providers customers can add accounts for.

        [API documentation: getOrganizationCellularGatewayEsimsServiceProviders](https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-esims-service-providers)

        Args:
            organization_id: Organization ID.

        Returns:
            Successful operation.

        Note:
            Returns a lazy PaginatedResponse
            that can be iterated or collected with `.collect()`.
            Page metadata is available on `.meta` and `.meta_pages`.

        Example API response:
            ```json
            {
              "items": [
                {
                  "name": "AT&T",
                  "logo": {
                    "url": "Logo URL"
                  },
                  "isBootstrap": false,
                  "terms": {
                    "content": "Legal jargon",
                    "name": "AT&T Terms and Conditions"
                  }
                }
              ],
              "meta": {
                "counts": {
                  "items": {
                    "total": 42,
                    "remaining": 0
                  }
                }
              }
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders"

        return self._session.get_pages(
            scope="cellularGateway",
            operation_id="getOrganizationCellularGatewayEsimsServiceProviders",
            path=path,
            item_schema=GetOrganizationCellularGatewayEsimsServiceProvidersResponseItemsItem,
        )

    def get_organization_cellular_gateway_esims_service_providers_accounts(
        self, organization_id: str, *, account_ids: list[int] | None = None
    ) -> PaginatedResponse[GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponseItem]:
        """Inventory of service provider accounts tied to the organization.

        [API documentation: getOrganizationCellularGatewayEsimsServiceProvidersAccounts](https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-esims-service-providers-accounts)

        Args:
            organization_id: Organization ID.
            account_ids: Optional parameter to filter the results by service provider account IDs.

        Returns:
            Successful operation.

        Note:
            Returns a lazy PaginatedResponse
            that can be iterated or collected with `.collect()`.
            Page metadata is available on `.meta` and `.meta_pages`.

        Example API response:
            ```json
            [
              {
                "items": [
                  {
                    "accountId": "0987654321",
                    "lastUpdatedAt": "2023-08-21T00:00:00Z",
                    "serviceProvider": {
                      "name": "ATT",
                      "logo": {
                        "url": "serviceproviderlogo.url"
                      }
                    },
                    "title": "My AT&T account",
                    "username": "MerakiUser"
                  }
                ],
                "meta": {
                  "counts": {
                    "items": {
                      "total": 1,
                      "remaining": 0
                    }
                  }
                }
              }
            ]
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders/accounts"

        params: dict[str, Any] = {}
        if account_ids is not None:
            params["accountIds[]"] = account_ids

        return self._session.get_pages(
            scope="cellularGateway",
            operation_id="getOrganizationCellularGatewayEsimsServiceProvidersAccounts",
            path=path,
            params=params,
            item_schema=GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponseItem,
        )

    def create_organization_cellular_gateway_esims_service_providers_account(
        self,
        *,
        organization_id: str,
        account_id: str,
        api_key: str,
        service_provider: CreateOrganizationCellularGatewayEsimsServiceProvidersAccountServiceProvider,
        title: str,
        username: str,
    ) -> CreateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse:
        """Add a service provider account.

        [API documentation: createOrganizationCellularGatewayEsimsServiceProvidersAccount](https://developer.cisco.com/meraki/api-v1/#!create-organization-cellular-gateway-esims-service-providers-account)

        Args:
            organization_id: Organization ID.
            account_id: Service provider account ID.
            api_key: Service provider account API key.
            service_provider: Service Provider information.
            title: Service provider account name.
            username: Service provider account username.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "accountId": "0987654321",
              "lastUpdatedAt": "2023-08-21T00:00:00Z",
              "serviceProvider": {
                "name": "ATT",
                "logo": {
                  "url": "serviceproviderlogo.url"
                }
              },
              "title": "My AT&T account",
              "username": "MerakiUser"
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders/accounts"

        payload: dict[str, Any] = {}
        if account_id is not None:
            payload["accountId"] = account_id
        if api_key is not None:
            payload["apiKey"] = api_key
        if service_provider is not None:
            payload["serviceProvider"] = service_provider.model_dump(
                by_alias=True, exclude_none=True
            )
        if title is not None:
            payload["title"] = title
        if username is not None:
            payload["username"] = username

        return self._session.post(
            scope="cellularGateway",
            operation_id="createOrganizationCellularGatewayEsimsServiceProvidersAccount",
            path=path,
            json=payload,
            response_schema=CreateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse,
        )

    def get_organization_cellular_gateway_esims_service_providers_accounts_communication_plans(
        self, *, organization_id: str, account_ids: list[str]
    ) -> PaginatedResponse[
        GetOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlansResponseItemsItem
    ]:
        """The communication plans available for a given provider.

        [API documentation: getOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlans](https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-esims-service-providers-accounts-communication-plans)

        Args:
            organization_id: Organization ID.
            account_ids: Account IDs that communication plans will be fetched for.

        Returns:
            Successful operation.

        Note:
            Returns a lazy PaginatedResponse
            that can be iterated or collected with `.collect()`.
            Page metadata is available on `.meta` and `.meta_pages`.

        Example API response:
            ```json
            {
              "items": [
                {
                  "accountId": "some account ID",
                  "apns": [
                    {
                      "name": "Some APN"
                    }
                  ],
                  "name": "A communication plan"
                }
              ],
              "meta": {
                "counts": {
                  "items": {
                    "total": 2,
                    "remaining": 0
                  }
                }
              }
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders/accounts/communicationPlans"

        params: dict[str, Any] = {}
        if account_ids is not None:
            params["accountIds[]"] = account_ids

        return self._session.get_pages(
            scope="cellularGateway",
            operation_id="getOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlans",
            path=path,
            params=params,
            item_schema=GetOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlansResponseItemsItem,
        )

    def get_organization_cellular_gateway_esims_service_providers_accounts_rate_plans(
        self, *, organization_id: str, account_ids: list[str]
    ) -> PaginatedResponse[
        GetOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlansResponseItemsItem
    ]:
        """The rate plans available for a given provider.

        [API documentation: getOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlans](https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-esims-service-providers-accounts-rate-plans)

        Args:
            organization_id: Organization ID.
            account_ids: Account IDs that rate plans will be fetched for.

        Returns:
            Successful operation.

        Note:
            Returns a lazy PaginatedResponse
            that can be iterated or collected with `.collect()`.
            Page metadata is available on `.meta` and `.meta_pages`.

        Example API response:
            ```json
            {
              "items": [
                {
                  "accountId": "account ID",
                  "name": "A rate plan"
                }
              ],
              "meta": {
                "counts": {
                  "items": {
                    "total": 2,
                    "remaining": 0
                  }
                }
              }
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders/accounts/ratePlans"

        params: dict[str, Any] = {}
        if account_ids is not None:
            params["accountIds[]"] = account_ids

        return self._session.get_pages(
            scope="cellularGateway",
            operation_id="getOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlans",
            path=path,
            params=params,
            item_schema=GetOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlansResponseItemsItem,
        )

    def update_organization_cellular_gateway_esims_service_providers_account(
        self,
        *,
        organization_id: str,
        account_id: str,
        title: str | None = None,
        api_key: str | None = None,
    ) -> UpdateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse:
        """Edit service provider account info stored in Meraki's database.

        [API documentation: updateOrganizationCellularGatewayEsimsServiceProvidersAccount](https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-service-providers-account)

        Args:
            organization_id: Organization ID.
            account_id: Account ID.
            title: Service provider account name used on the Meraki UI.
            api_key: Service provider account API key.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "accountId": "0987654321",
              "lastUpdatedAt": "2023-08-21T00:00:00Z",
              "serviceProvider": {
                "name": "ATT",
                "logo": {
                  "url": "serviceproviderlogo.url"
                }
              },
              "title": "My AT&T account",
              "username": "MerakiUser"
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        account_id = urllib.parse.quote(str(account_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders/accounts/{account_id}"

        payload: dict[str, Any] = {}
        if title is not None:
            payload["title"] = title
        if api_key is not None:
            payload["apiKey"] = api_key

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateOrganizationCellularGatewayEsimsServiceProvidersAccount",
            path=path,
            json=payload,
            response_schema=UpdateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse,
        )

    def delete_organization_cellular_gateway_esims_service_providers_account(
        self, *, organization_id: str, account_id: str
    ) -> None:
        """Remove a service provider account's integration with the Dashboard.

        [API documentation: deleteOrganizationCellularGatewayEsimsServiceProvidersAccount](https://developer.cisco.com/meraki/api-v1/#!delete-organization-cellular-gateway-esims-service-providers-account)

        Args:
            organization_id: Organization ID.
            account_id: Account ID.

        Returns:
            Successful operation.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        account_id = urllib.parse.quote(str(account_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/serviceProviders/accounts/{account_id}"

        return self._session.delete(
            scope="cellularGateway",
            operation_id="deleteOrganizationCellularGatewayEsimsServiceProvidersAccount",
            path=path,
        )

    def create_organization_cellular_gateway_esims_swap(
        self,
        *,
        organization_id: str,
        swaps: list[CreateOrganizationCellularGatewayEsimsSwapSwapsItem],
    ) -> CreateOrganizationCellularGatewayEsimsSwapResponse:
        """Swap which profile an eSIM uses.

        [API documentation: createOrganizationCellularGatewayEsimsSwap](https://developer.cisco.com/meraki/api-v1/#!create-organization-cellular-gateway-esims-swap)

        Args:
            organization_id: Organization ID.
            swaps: Each object represents a swap for one eSIM.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "eid": "1234567890",
              "iccid": "9876543210",
              "status": "Completed"
            }
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/swap"

        payload: dict[str, Any] = {}
        if swaps is not None:
            payload["swaps"] = [item.model_dump(by_alias=True, exclude_none=True) for item in swaps]

        return self._session.post(
            scope="cellularGateway",
            operation_id="createOrganizationCellularGatewayEsimsSwap",
            path=path,
            json=payload,
            response_schema=CreateOrganizationCellularGatewayEsimsSwapResponse,
        )

    def update_organization_cellular_gateway_esims_swap(
        self, *, id: str, organization_id: str
    ) -> UpdateOrganizationCellularGatewayEsimsSwapResponse:
        """Get the status of a profile swap.

        [API documentation: updateOrganizationCellularGatewayEsimsSwap](https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-swap)

        Args:
            id: eSIM EID.
            organization_id: Organization ID.

        Returns:
            Successful operation.

        Example API response:
            ```json
            {
              "eid": "1234567890",
              "iccid": "9876543210",
              "status": "Completed"
            }
            ```

        """
        id = urllib.parse.quote(str(id), safe="")
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/esims/swap/{id}"

        return self._session.put(
            scope="cellularGateway",
            operation_id="updateOrganizationCellularGatewayEsimsSwap",
            path=path,
            response_schema=UpdateOrganizationCellularGatewayEsimsSwapResponse,
        )

    def get_organization_cellular_gateway_uplink_statuses(
        self,
        organization_id: str,
        *,
        per_page: int | None = None,
        starting_after: str | None = None,
        ending_before: str | None = None,
        network_ids: list[str] | None = None,
        serials: list[str] | None = None,
        iccids: list[str] | None = None,
        total_pages: int | Literal["all"] = "all",
        direction: Literal["prev", "next"] = "next",
    ) -> PaginatedResponse[GetOrganizationCellularGatewayUplinkStatusesResponseItem]:
        """List the uplink status of every Meraki MG cellular gateway in the organization.

        [API documentation: getOrganizationCellularGatewayUplinkStatuses](https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-uplink-statuses)

        Args:
            organization_id: Organization ID.
            per_page: The number of entries per page returned. Acceptable range is 3 - 1000. Default
                is 1000.
            starting_after: A token used by the server to indicate the start of the page. Often this
                is a timestamp or an ID but it is not limited to those. This parameter
                should not be defined by client applications. The link for the first,
                last, prev, or next page in the HTTP Link header should define it.
            ending_before: A token used by the server to indicate the end of the page. Often this is
                a timestamp or an ID but it is not limited to those. This parameter
                should not be defined by client applications. The link for the first,
                last, prev, or next page in the HTTP Link header should define it.
            network_ids: A list of network IDs. The returned devices will be filtered to only
                include these networks.
            serials: A list of serial numbers. The returned devices will be filtered to only include
                these serials.
            iccids: A list of ICCIDs. The returned devices will be filtered to only include these
                ICCIDs.
            total_pages: use with per_page to get total results up to total_pages * per_page; -1 or
                "all" for all pages.
            direction: direction to paginate, either "next" (default) or "prev" page.

        Returns:
            Successful operation.

        Note:
            Returns a lazy PaginatedResponse
            that can be iterated or collected with `.collect()`.
            Page metadata is available on `.meta` and `.meta_pages`.

        Example API response:
            ```json
            [
              {
                "networkId": "N_24329156",
                "serial": "Q234-ABCD-5678",
                "model": "MG21",
                "lastReportedAt": "2018-02-11T00:00:00Z",
                "uplinks": [
                  {
                    "interface": "cellular",
                    "status": "ready",
                    "ip": "1.2.3.4",
                    "provider": "at&t",
                    "publicIp": "123.123.123.1",
                    "model": "integrated",
                    "signalStat": {
                      "rsrp": "-120",
                      "rsrq": "-13"
                    },
                    "mcc": "123",
                    "mnc": "123",
                    "roaming": {
                      "status": "home"
                    },
                    "connectionType": "4g",
                    "apn": "internet",
                    "gateway": "100.100.100.100",
                    "dns1": "111.111.111.111",
                    "dns2": "222.222.222.222",
                    "signalType": "4G",
                    "mtu": 1500,
                    "iccid": "123456789",
                    "imsi": "123456789012345",
                    "msisdn": "123456789012345"
                  }
                ]
              }
            ]
            ```

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/cellularGateway/uplink/statuses"

        params: dict[str, Any] = {}
        if per_page is not None:
            params["perPage"] = per_page
        if starting_after is not None:
            params["startingAfter"] = starting_after
        if ending_before is not None:
            params["endingBefore"] = ending_before
        if network_ids is not None:
            params["networkIds[]"] = network_ids
        if serials is not None:
            params["serials[]"] = serials
        if iccids is not None:
            params["iccids[]"] = iccids

        return self._session.get_pages(
            scope="cellularGateway",
            operation_id="getOrganizationCellularGatewayUplinkStatuses",
            path=path,
            params=params,
            total_pages=total_pages,
            direction=direction,
            item_schema=GetOrganizationCellularGatewayUplinkStatusesResponseItem,
        )



"""Tests for wirelessController API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_organization_wireless_controller_availabilities_change_history(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_availabilities_change_history endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_availabilities_change_history(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_clients_overview_history_by_device_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_clients_overview_history_by_device_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_clients_overview_history_by_device_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_connections(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_connections endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_connections(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_l2_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_l2_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_l2_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_l2_statuses_change_history_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_l2_statuses_change_history_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_l2_statuses_change_history_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_l2_usage_history_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_l2_usage_history_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_l2_usage_history_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_l3_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_l3_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_l3_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_l3_statuses_change_history_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_l3_statuses_change_history_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_l3_statuses_change_history_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_l3_usage_history_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_l3_usage_history_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_l3_usage_history_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_packets_overview_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_packets_overview_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_packets_overview_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_interfaces_usage_history_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_interfaces_usage_history_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_interfaces_usage_history_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_redundancy_failover_history(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_redundancy_failover_history endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_redundancy_failover_history(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_redundancy_statuses(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_redundancy_statuses endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_redundancy_statuses(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_devices_system_utilization_history_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_devices_system_utilization_history_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_devices_system_utilization_history_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_controller_overview_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_controller_overview_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless_controller.get_organization_wireless_controller_overview_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)



"""Response schemas for campus_gateway endpoints.

This module is automatically @generated from the Meraki API specification.
Do not edit this file manually.
"""

from __future__ import annotations

from pydantic import Field

from meraki_client.schemas._base import _BaseSchema


class CampusGatewayAddressesItem(_BaseSchema):
    """Schema for CampusGatewayAddressesItem."""

    protocol: str | None = None
    address: str | None = None


class CampusGatewayAddressesItem2(_BaseSchema):
    """Schema for CampusGatewayAddressesItem2."""

    protocol: str
    address: str


class CampusGatewayAddressesItem3(_BaseSchema):
    """Schema for CampusGatewayAddressesItem3."""

    protocol: str | None = None
    address: str


class CampusGatewayAddressesItem4(_BaseSchema):
    """Schema for CampusGatewayAddressesItem4."""

    assignment_mode: str | None = Field(
        default=None, validation_alias="assignmentMode", serialization_alias="assignmentMode"
    )
    protocol: str | None = None
    gateway: str | None = None
    subnet_mask: str | None = Field(
        default=None, validation_alias="subnetMask", serialization_alias="subnetMask"
    )
    address: str | None = None


class CampusGatewayDevicesUplinksItem(_BaseSchema):
    """Schema for CampusGatewayDevicesUplinksItem."""

    interface: str | None = None
    addresses: list[CampusGatewayAddressesItem] = Field(default_factory=list)


class CampusGatewayNetwork(_BaseSchema):
    """Network details of the cluster."""

    id: str | None = None


class CampusGatewayTunnelsAddressesItem(_BaseSchema):
    """Schema for CampusGatewayTunnelsAddressesItem."""

    protocol: str | None = None
    gateway: str | None = None
    subnet_mask: str | None = Field(
        default=None, validation_alias="subnetMask", serialization_alias="subnetMask"
    )


class CampusGatewayTunnelsUplink(_BaseSchema):
    """Use uplink interface as tunnel interface."""

    interface: str | None = None


class CampusGatewayUplink(_BaseSchema):
    """Uplink setting of the cluster."""

    interface: str | None = None
    vlan: int | None = None
    addresses: list[CampusGatewayAddressesItem4] = Field(default_factory=list)


class CampusGatewayUplinksAddressesItem(_BaseSchema):
    """Schema for CampusGatewayUplinksAddressesItem."""

    assignment_mode: str | None = Field(
        default=None, validation_alias="assignmentMode", serialization_alias="assignmentMode"
    )
    protocol: str | None = None
    gateway: str | None = None
    subnet_mask: str | None = Field(
        default=None, validation_alias="subnetMask", serialization_alias="subnetMask"
    )


class CreateNetworkCampusGatewayClusterDevicesItem(_BaseSchema):
    """Item schema for devices."""

    serial: str
    uplinks: list[CreateNetworkCampusGatewayClusterDevicesItemUplinksItem] = Field(
        default_factory=list
    )
    tunnels: list[CreateNetworkCampusGatewayClusterDevicesItemUplinksItem] = Field(
        default_factory=list
    )


class CreateNetworkCampusGatewayClusterDevicesItemUplinksItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterDevicesItemUplinksItem."""

    interface: str
    addresses: list[CampusGatewayAddressesItem2]


class CreateNetworkCampusGatewayClusterNameservers(_BaseSchema):
    """Nameservers of the cluster."""

    addresses: list[str] = Field(default_factory=list)


class CreateNetworkCampusGatewayClusterPortChannelsItem(_BaseSchema):
    """Item schema for portChannels."""

    name: str
    vlan: int
    allowed_vlans: str = Field(validation_alias="allowedVlans", serialization_alias="allowedVlans")


class CreateNetworkCampusGatewayClusterResponse(_BaseSchema):
    """Response for createNetworkCampusGatewayCluster operation."""

    cluster_id: str | None = Field(
        default=None, validation_alias="clusterId", serialization_alias="clusterId"
    )
    name: str | None = None
    uplinks: list[CreateNetworkCampusGatewayClusterResponseUplinksItem] = Field(
        default_factory=list
    )
    tunnels: list[CreateNetworkCampusGatewayClusterResponseTunnelsItem] = Field(
        default_factory=list
    )
    nameservers: CreateNetworkCampusGatewayClusterResponseNameservers | None = None
    port_channels: list[CreateNetworkCampusGatewayClusterResponsePortChannelsItem] = Field(
        default_factory=list, validation_alias="portChannels", serialization_alias="portChannels"
    )
    devices: list[CreateNetworkCampusGatewayClusterResponseDevicesItem] = Field(
        default_factory=list
    )
    notes: str | None = None
    url: str | None = None


class CreateNetworkCampusGatewayClusterResponseDevicesItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterResponseDevicesItem."""

    serial: str | None = None
    member_id: str | None = Field(
        default=None, validation_alias="memberId", serialization_alias="memberId"
    )
    uplinks: list[CampusGatewayDevicesUplinksItem] = Field(default_factory=list)
    tunnels: list[CampusGatewayDevicesUplinksItem] = Field(default_factory=list)


class CreateNetworkCampusGatewayClusterResponseNameservers(_BaseSchema):
    """Nameservers of the cluster."""

    addresses: list[str] = Field(default_factory=list)


class CreateNetworkCampusGatewayClusterResponsePortChannelsItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterResponsePortChannelsItem."""

    id: str | None = None
    name: str | None = None
    vlan: int | None = None
    allowed_vlans: str | None = Field(
        default=None, validation_alias="allowedVlans", serialization_alias="allowedVlans"
    )


class CreateNetworkCampusGatewayClusterResponseTunnelsItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterResponseTunnelsItem."""

    uplink: CampusGatewayTunnelsUplink | None = None
    interface: str | None = None
    vlan: int | None = None
    addresses: list[CampusGatewayTunnelsAddressesItem] = Field(default_factory=list)


class CreateNetworkCampusGatewayClusterResponseUplinksItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterResponseUplinksItem."""

    interface: str | None = None
    vlan: int | None = None
    addresses: list[CampusGatewayUplinksAddressesItem] = Field(default_factory=list)


class CreateNetworkCampusGatewayClusterTunnelsItem(_BaseSchema):
    """Item schema for tunnels."""

    uplink: CreateNetworkCampusGatewayClusterTunnelsItemUplink | None = None
    interface: str | None = None
    vlan: int | None = None
    addresses: list[CreateNetworkCampusGatewayClusterTunnelsItemAddressesItem] = Field(
        default_factory=list
    )


class CreateNetworkCampusGatewayClusterTunnelsItemAddressesItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterTunnelsItemAddressesItem."""

    protocol: str
    gateway: str
    subnet_mask: str = Field(validation_alias="subnetMask", serialization_alias="subnetMask")


class CreateNetworkCampusGatewayClusterTunnelsItemUplink(_BaseSchema):
    """Uplink interface details if uplink is reused as tunnel."""

    interface: str


class CreateNetworkCampusGatewayClusterUplinksItem(_BaseSchema):
    """Item schema for uplinks."""

    interface: str
    vlan: int
    addresses: list[CreateNetworkCampusGatewayClusterUplinksItemAddressesItem]


class CreateNetworkCampusGatewayClusterUplinksItemAddressesItem(_BaseSchema):
    """Schema for CreateNetworkCampusGatewayClusterUplinksItemAddressesItem."""

    assignment_mode: str = Field(
        validation_alias="assignmentMode", serialization_alias="assignmentMode"
    )
    protocol: str | None = None
    gateway: str | None = None
    subnet_mask: str | None = Field(
        default=None, validation_alias="subnetMask", serialization_alias="subnetMask"
    )


class GetOrganizationCampusGatewayClustersResponseItemsItem(_BaseSchema):
    """Schema for GetOrganizationCampusGatewayClustersResponseItemsItem."""

    network: CampusGatewayNetwork | None = None
    cluster_id: str | None = Field(
        default=None, validation_alias="clusterId", serialization_alias="clusterId"
    )
    name: str | None = None
    uplinks: list[CreateNetworkCampusGatewayClusterResponseUplinksItem] = Field(
        default_factory=list
    )
    tunnels: list[CreateNetworkCampusGatewayClusterResponseTunnelsItem] = Field(
        default_factory=list
    )
    nameservers: CreateNetworkCampusGatewayClusterResponseNameservers | None = None
    port_channels: list[CreateNetworkCampusGatewayClusterResponsePortChannelsItem] = Field(
        default_factory=list, validation_alias="portChannels", serialization_alias="portChannels"
    )
    devices: list[CreateNetworkCampusGatewayClusterResponseDevicesItem] = Field(
        default_factory=list
    )
    notes: str | None = None
    url: str | None = None


class GetOrganizationCampusGatewayDevicesUplinksLocalOverridesByDeviceResponseItemsItem(
    _BaseSchema
):
    """Schema for
    GetOrganizationCampusGatewayDevicesUplinksLocalOverridesByDeviceResponseItemsItem.
    """

    serial: str | None = None
    uplink: CampusGatewayUplink | None = None
    nameservers: CreateNetworkCampusGatewayClusterResponseNameservers | None = None
    sgt: int | None = None


class UpdateNetworkCampusGatewayClusterDevicesItem(_BaseSchema):
    """Item schema for devices."""

    serial: str
    uplinks: list[UpdateNetworkCampusGatewayClusterDevicesItemUplinksItem] = Field(
        default_factory=list
    )
    tunnels: list[UpdateNetworkCampusGatewayClusterDevicesItemUplinksItem] = Field(
        default_factory=list
    )


class UpdateNetworkCampusGatewayClusterDevicesItemUplinksItem(_BaseSchema):
    """Schema for UpdateNetworkCampusGatewayClusterDevicesItemUplinksItem."""

    interface: str
    addresses: list[CampusGatewayAddressesItem3]


class UpdateNetworkCampusGatewayClusterNameservers(_BaseSchema):
    """Nameservers of the cluster."""

    addresses: list[str] = Field(default_factory=list)


class UpdateNetworkCampusGatewayClusterPortChannelsItem(_BaseSchema):
    """Item schema for portChannels."""

    name: str
    vlan: int | None = None
    allowed_vlans: str | None = Field(
        default=None, validation_alias="allowedVlans", serialization_alias="allowedVlans"
    )


class UpdateNetworkCampusGatewayClusterResponse(_BaseSchema):
    """Response for updateNetworkCampusGatewayCluster operation."""

    cluster_id: str | None = Field(
        default=None, validation_alias="clusterId", serialization_alias="clusterId"
    )
    name: str | None = None
    uplinks: list[CreateNetworkCampusGatewayClusterResponseUplinksItem] = Field(
        default_factory=list
    )
    tunnels: list[CreateNetworkCampusGatewayClusterResponseTunnelsItem] = Field(
        default_factory=list
    )
    nameservers: CreateNetworkCampusGatewayClusterResponseNameservers | None = None
    port_channels: list[CreateNetworkCampusGatewayClusterResponsePortChannelsItem] = Field(
        default_factory=list, validation_alias="portChannels", serialization_alias="portChannels"
    )
    devices: list[CreateNetworkCampusGatewayClusterResponseDevicesItem] = Field(
        default_factory=list
    )
    notes: str | None = None
    url: str | None = None


class UpdateNetworkCampusGatewayClusterTunnelsItem(_BaseSchema):
    """Item schema for tunnels."""

    uplink: CreateNetworkCampusGatewayClusterTunnelsItemUplink | None = None
    interface: str | None = None
    vlan: int | None = None
    addresses: list[UpdateNetworkCampusGatewayClusterTunnelsItemAddressesItem] = Field(
        default_factory=list
    )


class UpdateNetworkCampusGatewayClusterTunnelsItemAddressesItem(_BaseSchema):
    """Schema for UpdateNetworkCampusGatewayClusterTunnelsItemAddressesItem."""

    protocol: str | None = None
    gateway: str | None = None
    subnet_mask: str | None = Field(
        default=None, validation_alias="subnetMask", serialization_alias="subnetMask"
    )


class UpdateNetworkCampusGatewayClusterUplinksItem(_BaseSchema):
    """Item schema for uplinks."""

    interface: str
    vlan: int | None = None
    addresses: list[UpdateNetworkCampusGatewayClusterUplinksItemAddressesItem] = Field(
        default_factory=list
    )


class UpdateNetworkCampusGatewayClusterUplinksItemAddressesItem(_BaseSchema):
    """Schema for UpdateNetworkCampusGatewayClusterUplinksItemAddressesItem."""

    assignment_mode: str | None = Field(
        default=None, validation_alias="assignmentMode", serialization_alias="assignmentMode"
    )
    protocol: str | None = None
    gateway: str | None = None
    subnet_mask: str | None = Field(
        default=None, validation_alias="subnetMask", serialization_alias="subnetMask"
    )



"""Shared constants for code generation."""

import os

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
TEMPLATE_DIR = os.path.join(SCRIPT_DIR, "templates")
SPEC_OVERRIDES_FILE = os.path.join(SCRIPT_DIR, "spec_overrides.toml")

# Python keywords and builtins that cannot be used as parameter names
RESERVED_NAMES = {
    "and",
    "as",
    "assert",
    "async",
    "await",
    "break",
    "class",
    "continue",
    "def",
    "del",
    "elif",
    "else",
    "except",
    "finally",
    "for",
    "from",
    "global",
    "if",
    "import",
    "in",
    "is",
    "lambda",
    "nonlocal",
    "not",
    "or",
    "pass",
    "raise",
    "return",
    "try",
    "while",
    "with",
    "yield",
    "type",
    "list",
    "dict",
    "set",
    "str",
    "int",
    "float",
    "bool",
    "object",
    "filter",
    "format",
    "hash",
    "input",
    "open",
    "range",
    "zip",
}



"""Tests for spaces API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_organization_spaces_integrate_status(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_spaces_integrate_status endpoint."""
    with skip_on_unsupported():
        client.spaces.get_organization_spaces_integrate_status(organization_id=organization_id)
